#include "20mhz_delays.h"
#include "ds18b20.h"

bit onewire_reset(void) {

	bit present @ bit_holder.1;

	DQ_TRI = 1;
	DQ_PIN = 0;
	DQ_TRI = 0;							// pull low for 500us pulse
	delay_100us();
	delay_100us();
	delay_100us();
	delay_100us();
	delay_100us();
	DQ_TRI = 1;

	delay_50us();
	delay_5us();
	delay_5us();
	delay_5us();

	present = DQ_PIN;					// read line

	delay_100us();
	delay_100us();
	delay_100us();
	delay_100us();

	if(present)
		return 0;
	else
		return 1;

}

void onewire_write(uns8 payload) {

	uns8 count;

	for(count = 0; count < 8; count++) {

		DQ_PIN = 0;
		DQ_TRI = 0;						// pull low for 2us

		delay_3us();

		if(payload.0)					// send bit
			DQ_TRI = 1;
		else
			DQ_TRI = 0;

		delay_50us();
		delay_50us();

		DQ_TRI = 1;						// return to idle state

		delay_5us();
		delay_5us();

		payload = payload >> 1;
	}

	delay_5us();

}

uns8 onewire_read(void) {

	uns8 count,
		 payload = 0;

	bit DQ_data @ bit_holder.2;

	for(count = 0; count < 8; count++) {

		DQ_PIN = 0;
		DQ_TRI = 0;

		delay_3us();					// hold low for 3us

		DQ_TRI = 1;						// release line

		delay_5us();					// wait 10us
		delay_5us();

		DQ_data = DQ_PIN;				// sample data pin

		if(count != 8)
			payload = payload >> 1;		// shift data byte

		if(DQ_data)						// if data = 1, add 0x80 to data byte
			payload.7 = 1;

		delay_50us();					// finish read slot
		delay_5us();

	}

	delay_5us();						// minimum 1us between reads

	return payload;

}

void onewire_start_conv(bit parasite_power) {

	onewire_reset();
	onewire_write(SKIP_ROM);				// skip rom
	onewire_write(START_CONV);				// start conversion

	if(parasite_power) {
		DQ_TRI = 0;
		DQ_PIN = 1;
	}

	delay_ms(100);

}

uns8 ds18b20_read_temp(uns8 sensorID) {

	uns8 i, storage, temp[2];

	onewire_reset();
	onewire_write(MATCH_ROM);

	for(i = 0; i < 8; i++) {
		switch (sensorID) {
			case 0:		onewire_write(DS18B20_ID_0[i]); break;
			case 1:		onewire_write(DS18B20_ID_1[i]); break;
			case 2:		onewire_write(DS18B20_ID_2[i]); break;
			case 3:		onewire_write(DS18B20_ID_3[i]); break;
			case 4:		onewire_write(DS18B20_ID_4[i]); break;
			case 5:		onewire_write(DS18B20_ID_5[i]); break;
			case 6:		onewire_write(DS18B20_ID_6[i]); break;
			case 7:		onewire_write(DS18B20_ID_7[i]); break;
			case 8:		onewire_write(DS18B20_ID_8[i]); break;
			case 9:		onewire_write(DS18B20_ID_9[i]); break;
		}
	}

	onewire_write(READ_SCRATCHPAD);

	for(i = 0; i < 2; i++) {			// only need first two bytes
		storage = onewire_read();
		temp[i] = storage;
	}

	if(temp[0] & 0x08)
		half_degree = 1;
	else
		half_degree = 0;

	temp[0] &= 0xF0;					// zero all decimal values
	storage = swap(temp[0]);			// realign temperature bits

	if(temp[1] & 0x01)
		storage += 16;
	if(temp[1] & 0x02)
		storage += 32;
	if(temp[1] & 0x04)
		storage += 64;
	if(temp[1] & 0x80)					// negative?
		storage += 128;

	onewire_reset();

	return storage;

}
