// DS18B20 Library
// For 16F877A @ 20MHz

#ifndef __DS18B20_H
#define __DS18B20_H

#define DQ_TRI TRISB.2
#define DQ_PIN PORTB.2

#define READ_ROM 0x33
#define MATCH_ROM 0x55
#define SKIP_ROM 0xCC
#define START_CONV 0x44
#define READ_SCRATCHPAD 0xBE

static const uns8 DS18B20_ID_0[8] = {0x28, 0x93, 0xE5, 0xDF, 0x02, 0x00, 0x00, 0x2A},
				  DS18B20_ID_1[8] = {0x28, 0xF8, 0xCF, 0xDF, 0X02, 0x00, 0x00, 0x54},
				  DS18B20_ID_2[8] = {0x28, 0x9D, 0x46, 0xE1, 0x02, 0x00, 0x00, 0x19},
				  DS18B20_ID_3[8] = {0x28, 0x47, 0x5A, 0xE1, 0x02, 0x00, 0x00, 0x4F},
				  DS18B20_ID_4[8] = {0x28, 0x2C, 0x5D, 0xE1, 0x02, 0x00, 0x00, 0x25},
				  DS18B20_ID_5[8] = {0x28, 0xCA, 0x6B, 0xE1, 0x02, 0x00, 0x00, 0xA6},
				  DS18B20_ID_6[8] = {0x28, 0x97, 0x7B, 0xE1, 0x02, 0x00, 0x00, 0xBE},
				  DS18B20_ID_7[8] = {0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},	// not set
				  DS18B20_ID_8[8] = {0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},	// not set
				  DS18B20_ID_9[8] = {0x28, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};	// not set

uns8 bit_holder;
bit half_degree @ bit_holder.0;	// bit to indicate half degree present on ds18b20 reading

bit onewire_reset(void);		// resets OneWire bus, returns 1 if one or more presence pulses were detected and 0 otherwise
void onewire_write(uns8);		// writes one byte to OneWire bus
uns8 onewire_read(void);		// reads one byte from OneWire bus
void onewire_start_conv(bit);	// starts a temperature conversion, takes 93.75ms max for 9-bit conversion
uns8 ds18b20_read_temp(uns8);	// returns the temperature in the format of unsigned byte with the MSB=1 indicating negative.

#endif
